@@ -11,7 +11,16 @@ from rich.box import SIMPLE
 from InquirerPy import inquirer
 from InquirerPy.base.control import Choice
 from InquirerPy.validator import PathValidator
-from ..core.ui_control import theme
+from ..ui.common_ui import CommonUI, Message
+
+
+# Import theme system for colors
+from ..ui.theme import get_color
+
+WARNING_COLOR = get_color('warning')
+PRIMARY_COLOR = get_color('primary')
+ACCENT_COLOR = get_color('accent')
+SUCCESS_COLOR = get_color('success')
 
 ROSE_BANNER = """
 ██████╗  ██████╗ ███████╗███████╗
@@ -28,21 +37,21 @@ def build_banner():
     title = Text()
     title.append("ROS Bag Filter Tool") 
     subtitle = Text()
-    subtitle.append("Github", style=f"{theme.WARNING} link https://github.com/hanxiaomax/rose")
+    subtitle.append("Github", style=f"{get_color('primary')} link https://github.com/hanxiaomax/rose")
     subtitle.append(" • ", style="dim")
-    subtitle.append("Author", style=f"{theme.WARNING} link https://github.com/hanxiaomax")
+    subtitle.append("Author", style=f"{get_color('primary')} link https://github.com/hanxiaomax")
 
     # Create banner content
     content = Text()
-    content.append(ROSE_BANNER, style="")
-    content.append("Yet another cross-platform and ROS Environment independent editor/filter tool for ROS bag files", style=f"dim {theme.PRIMARY}")
+    content.append(ROSE_BANNER, style=f"{get_color('primary')}")
+    content.append("ROSE is a Ros Bag One-Stop Editor", style=f"{get_color('primary')}")
     
     # Create panel with all elements
     panel = Panel(
         content,
         title=title,
         subtitle=subtitle,  
-        border_style=theme.WARNING,  
+        border_style=get_color('accent'),  # Use Claude's signature color
         highlight=True
     )
     
@@ -51,16 +60,20 @@ def build_banner():
     return panel
   
 def print_usage_instructions(console:Console, is_fuzzy:bool = False):
-    console.print("\nUsage Instructions:",style=f"bold {theme.ACCENT}")
+    from ..ui.common_ui import CommonUI
+    ui = CommonUI()
+    ui.console = console
+    
+    Message.accent("Usage Instructions:", console)
     if is_fuzzy:
-        console.print(f"•  [{theme.ACCENT}]Type to search[/{theme.ACCENT}]")
+        Message.accent("•  Type to search", console)
     else:
-        console.print(f"•  [{theme.ACCENT}]Space[/{theme.ACCENT}] to select/unselect") 
-    console.print(f"•  [{theme.ACCENT}]↑/↓[/{theme.ACCENT}] to navigate options")
-    console.print(f"•  [{theme.ACCENT}]Tab[/{theme.ACCENT}] to select and move to next item")
-    console.print(f"•  [{theme.ACCENT}]Shift+Tab[/{theme.ACCENT}] to select and move to previous item")
-    console.print(f"•  [{theme.ACCENT}]Ctrl+A[/{theme.ACCENT}] to select all")
-    console.print(f"•  [{theme.ACCENT}]Enter[/{theme.ACCENT}] to confirm selection\n")
+        Message.accent("•  Space to select/unselect", console) 
+    Message.accent("•  ↑/↓ to navigate options", console)
+    Message.accent("•  Tab to select and move to next item", console)
+    Message.accent("•  Shift+Tab to select and move to previous item", console)
+    Message.accent("•  Ctrl+A to select all", console)
+    Message.accent("•  Enter to confirm selection\n", console)
 
 
 def collect_bag_files(directory: str) -> List[str]:
@@ -80,19 +93,19 @@ def print_bag_info(console:Console, bag_path: str, topics: List[str], connection
     
     # Create basic bag info text
     bag_info = Text()
-    bag_info.append(f"File: {os.path.basename(bag_path)}\n", style=f"bold {theme.ACCENT}")
-    bag_info.append(f"Size: {file_size_mb:.2f} MB ({file_size:,} bytes)\n",style=f"dim {theme.PRIMARY}")
-    bag_info.append(f"Path: {os.path.abspath(bag_path)}\n",style=f"dim {theme.PRIMARY}")
-    bag_info.append(f"Topics({len(topics)} in total):\n", style=theme.ACCENT)
+    bag_info.append(f"File: {os.path.basename(bag_path)}\n", style=f"bold {Theme.get_color('accent')}")
+    bag_info.append(f"Size: {file_size_mb:.2f} MB ({file_size:,} bytes)\n",style=f"dim {Theme.get_color('primary')}")
+    bag_info.append(f"Path: {os.path.abspath(bag_path)}\n",style=f"dim {Theme.get_color('primary')}")
+    bag_info.append(f"Topics({len(topics)} in total):\n", style=Theme.get_color('accent'))
     
     # First, display all topics
     for topic in sorted(topics):
-        bag_info.append(f"• {topic:<40}", style=f"{theme.PRIMARY}")
-        bag_info.append(f"{connections[topic]}\n", style=f"dim {theme.PRIMARY}")
+        bag_info.append(f"• {topic:<40}", style=f"{Theme.get_color('primary')}")
+        bag_info.append(f"{connections[topic]}\n", style=f"dim {Theme.get_color('primary')}")
     
     panel = Panel(bag_info,
                   title=f"Bag Information",
-                  border_style=theme.ACCENT,
+                  border_style=Theme.get_color('accent'),
                   padding=(0, 1))
     
     console.print(panel)
@@ -102,10 +115,9 @@ def print_bag_info(console:Console, bag_path: str, topics: List[str], connection
         action = inquirer.select(
             message="What would you like to do?",
             choices=[
-                Choice(value="filter", name="1. Filter topics (fuzzy search)"),
+                Choice(value="filter", name="1. show topics (fuzzy search)"),
                 Choice(value="back", name="2. Back")
-            ],
-            style=theme.get_inquirer_style()
+            ]
         ).execute()
         
         if action == "back":
@@ -115,23 +127,23 @@ def print_bag_info(console:Console, bag_path: str, topics: List[str], connection
             filtered_topics = ask_topics(console, topics, parser=parser, bag_path=bag_path)
             
             if not filtered_topics:
-                console.print("No topics selected. Showing all topics.", style=theme.WARNING)
+                console.print("No topics selected. Showing all topics.", style=Theme.get_color('warning'))
                 continue
             
             # Create filtered topics panel
             filtered_info = Text()
-            filtered_info.append(f"File: {os.path.basename(bag_path)}\n", style=f"bold {theme.ACCENT}")
+            filtered_info.append(f"File: {os.path.basename(bag_path)}\n", style=f"bold {Theme.get_color('accent')}")
             filtered_info.append(f"Size: {file_size_mb:.2f} MB ({file_size:,} bytes)\n")
             filtered_info.append(f"Path: {os.path.abspath(bag_path)}\n")
             filtered_info.append(f"Filtered Topics({len(filtered_topics)} of {len(topics)}):\n", style="bold")
             
             for topic in sorted(filtered_topics):
-                filtered_info.append(f"• {topic:<40}", style=theme.PRIMARY)
+                filtered_info.append(f"• {topic:<40}", style=Theme.get_color('primary'))
                 filtered_info.append(f"{connections[topic]}\n", style="dim")
             
             filtered_panel = Panel(filtered_info,
                                   title=f"Filtered Bag Information",
-                                  border_style=theme.PRIMARY,
+                                  border_style=Theme.get_color('primary'),
                                   padding=(0, 1))
             
             console.print(filtered_panel)
@@ -140,12 +152,12 @@ def print_filter_stats(console:Console, input_bag: str, output_bag: str):
     """Show filtering statistics in a table format with headers and three rows"""
     # Check if files exist before trying to get their sizes
     if not os.path.exists(input_bag):
-        console.print(f"[{theme.ERROR}]✗ Input bag file not found: {input_bag}[/{theme.ERROR}]")
+        ErrorMessage(f"Input bag file not found: {input_bag}").render(console)
         return
     
     if not os.path.exists(output_bag):
-        console.print(f"[{theme.WARNING}]⚠ Output bag file not created: {output_bag}[/{theme.WARNING}]")
-        console.print(f"[{theme.WARNING}]The filtering process may have failed or been interrupted.[/{theme.WARNING}]")
+        WarningMessage(f"Output bag file not created: {output_bag}").render(console)
+        WarningMessage("The filtering process may have failed or been interrupted.").render(console)
         return
     
     input_size = os.path.getsize(input_bag)
@@ -164,9 +176,9 @@ def print_filter_stats(console:Console, input_bag: str, output_bag: str):
     )
     
     # 添加三列，第一列较窄
-    table.add_column("", style=f"{theme.PRIMARY}", width=4)  # 用于input/output标签
-    table.add_column("file", style=f"{theme.PRIMARY}", justify="left")  # 文件名列
-    table.add_column("size", style=f"{theme.PRIMARY}", justify="left", width=20)  # 增加size列宽度以适应额外信息
+    table.add_column("", style=f"{Theme.get_color('primary')}", width=4)  # 用于input/output标签
+    table.add_column("file", style=f"{Theme.get_color('primary')}", justify="left")  # 文件名列
+    table.add_column("size", style=f"{Theme.get_color('primary')}", justify="left", width=20)  # 增加size列宽度以适应额外信息
     
     # 添加input行
     table.add_row(
@@ -177,13 +189,13 @@ def print_filter_stats(console:Console, input_bag: str, output_bag: str):
     
     # 添加output行，包含缩小百分比，使用ACCENT颜色
     table.add_row(
-        f"[{theme.ACCENT}]Out[/{theme.ACCENT}]",
-        f"[{theme.ACCENT}]{os.path.basename(output_bag)}[/{theme.ACCENT}]",
-        f"[{theme.ACCENT}]{output_size_mb:.0f}MB (↓{reduction_ratio:.0f}%)[/{theme.ACCENT}]"
+        f"[{Theme.get_color('accent')}]Out[/{Theme.get_color('accent')}]",
+        f"[{Theme.get_color('accent')}]{os.path.basename(output_bag)}[/{Theme.get_color('accent')}]", 
+        f"[{Theme.get_color('accent')}]{output_size_mb:.0f}MB (↓{reduction_ratio:.0f}%)[/{Theme.get_color('accent')}]"  
     )
     
     # 将表格放在面板中显示
-    console.print(Panel(table, title="Filter Results", border_style=f"bold {theme.ACCENT}"))
+    console.print(Panel(table, title="Filter Results", border_style=f"bold {Theme.get_color('accent')}"))
 
 def print_batch_filter_summary(console:Console, success_count: int, fail_count: int):
     """Show filtering results for batch processing
@@ -202,9 +214,9 @@ def print_batch_filter_summary(console:Console, success_count: int, fail_count:
     )
     
     if fail_count == 0:
-        console.print(summary, style=f"{theme.SUCCESS}")
+        console.print(summary, style=Theme.get_color('success'))
     else:
-        console.print(summary, style=f"{theme.ACCENT}")
+        console.print(summary, style=Theme.get_color('accent'))
 
 def ask_topics(console: Console, topics: List[str], parser=None, bag_path: Optional[str] = None) -> Optional[List[str]]:
     return ask_topics_with_fuzzy(
@@ -217,6 +229,68 @@ def ask_topics(console: Console, topics: List[str], parser=None, bag_path: Optio
         bag_path=bag_path
     )
 
+def filter_topics(all_topics: List[str], patterns: List[str], topic_filter: Optional[str] = None) -> List[str]:
+    """
+    Filter topics based on patterns and optional topic filter with fuzzy matching support
+    
+    Args:
+        all_topics: List of all available topics
+        patterns: List of patterns to match (supports fuzzy matching)
+        topic_filter: Optional additional filter pattern
+        
+    Returns:
+        List of matching topics
+        
+    Examples:
+        filter_topics(['/gps/fix', '/imu/data', '/camera/image'], ['gps']) -> ['/gps/fix']
+        filter_topics(['/gps/fix', '/imu/data'], ['gps', 'imu']) -> ['/gps/fix', '/imu/data']
+        filter_topics(['/tf', '/tf_static'], ['^/tf$']) -> ['/tf']  # regex exact match
+    """
+    if not patterns:
+        return all_topics
+    
+    import re
+    matching_topics = set()
+    
+    for pattern in patterns:
+        # Exact match first (highest priority)
+        if pattern in all_topics:
+            matching_topics.add(pattern)
+            continue
+        
+        # Fuzzy matching - if pattern is a substring of topic name (case insensitive)
+        for topic in all_topics:
+            if pattern.lower() in topic.lower():
+                matching_topics.add(topic)
+        
+        # Regex matching if pattern looks like a regex (contains regex special chars)
+        if any(char in pattern for char in ['^', '$', '*', '+', '?', '[', ']', '(', ')', '|', '\\']):
+            try:
+                regex = re.compile(pattern)
+                for topic in all_topics:
+                    if regex.search(topic):
+                        matching_topics.add(topic)
+            except re.error:
+                # Not a valid regex, skip regex matching for this pattern
+                pass
+    
+    # Apply additional topic filter if provided
+    if topic_filter:
+        filtered_topics = set()
+        try:
+            filter_regex = re.compile(topic_filter)
+            for topic in matching_topics:
+                if filter_regex.search(topic):
+                    filtered_topics.add(topic)
+            matching_topics = filtered_topics
+        except re.error:
+            # If regex is invalid, use substring matching
+            filtered_topics = {topic for topic in matching_topics if topic_filter.lower() in topic.lower()}
+            matching_topics = filtered_topics
+    
+    return sorted(list(matching_topics))
+
+
 def ask_topics_with_fuzzy(
     console: Console, 
     topics: List[str], 
@@ -297,7 +371,6 @@ def ask_topics_with_fuzzy(
         marker="● ",
         border=True,
         cycle=True,
-        style=theme.get_inquirer_style(),
         default=preselected
     ).execute()
     
@@ -332,7 +405,7 @@ class TimedPanelProgress(PanelProgress):
         # Display loading time after completion
         if self.start_time and self.end_time:
             elapsed = self.end_time - self.start_time
-            self._external_console.print(f"[{theme.SUCCESS}]✓ Bag file loaded in {elapsed:.2f}s[/{theme.SUCCESS}]")
+            self._external_console.print(f"[{SUCCESS_COLOR}]✓ Bag file loaded in {elapsed:.2f}s[/{SUCCESS_COLOR}]")
         
         return result
 
@@ -347,8 +420,8 @@ def LoadingAnimation(title: Optional[str] = None, dismiss: bool = False):
         PanelProgress: A progress bar wrapped in a panel with optional title
     """
     return PanelProgress(
-        TextColumn("[progress.description]{task.description}"),
-        BarColumn(bar_width=None),  # 设置为 None 以自适应宽度
+        TextColumn(f"[{Theme.get_color('primary')}][progress.description]{{task.description}}[/{Theme.get_color('primary')}]"),
+        BarColumn(bar_width=None, complete_style=Theme.get_color('success'), finished_style=Theme.get_color('success')),  # 设置为 None 以自适应宽度
         TaskProgressColumn(),
         TimeRemainingColumn(),
         title=title,
@@ -366,11 +439,124 @@ def LoadingAnimationWithTimer(title: Optional[str] = None, dismiss: bool = False
         TimedPanelProgress: A progress bar wrapped in a panel with timing functionality
     """
     return TimedPanelProgress(
-        TextColumn("[progress.description]{task.description}"),
-        BarColumn(bar_width=None),
+        TextColumn(f"[{Theme.get_color('primary')}][progress.description]{{task.description}}[/{Theme.get_color('primary')}]"),
+        BarColumn(bar_width=None, complete_style=Theme.get_color('success'), finished_style=Theme.get_color('success')),
         TaskProgressColumn(),
         TimeElapsedColumn(),
         title=title,
         transient=dismiss,
     )
 
+
+def check_and_load_bag_cache(bag_path: Path, auto_load: bool = True, verbose: bool = False, build_index: bool = False, force_load: bool = False) -> bool:
+    """
+    Check if bag is in cache, and optionally prompt user to load it if not
+    
+    Args:
+        bag_path: Path to the bag file
+        auto_load: Whether to prompt user for auto-loading
+        verbose: Whether to show verbose output
+        build_index: Whether to build DataFrame index when loading
+        force_load: Whether to force loading without prompting user
+    
+    Returns:
+        bool: True if bag is available in cache (either was already cached or was loaded), False otherwise
+    """
+    from ..core.cache import create_bag_cache_manager
+    from ..core.parser import BagParser
+    import asyncio
+    import typer
+    
+    # Check if bag is already loaded in cache
+    cache_manager = create_bag_cache_manager()
+    cached_entry = cache_manager.get_analysis(bag_path)
+    
+    if cached_entry and cached_entry.is_valid(bag_path):
+        if verbose:
+            console = Console()
+            from ..ui.common_ui import CommonUI
+            ui = CommonUI()
+            ui.console = console
+            Message.success(f"✓ Using cached bag analysis for {bag_path}")
+        return True
+    
+    if not auto_load and not force_load:
+        return False
+    
+    # Bag not in cache, ask user if they want to load it (unless force_load is True)
+    ui = CommonUI()
+    console = ui.console
+    
+    should_load = force_load
+    if not force_load:
+        from ..ui.common_ui import CommonUI
+        ui = CommonUI()
+        ui.console = console
+        Message.warning(f"⚠ Bag file {bag_path} is not loaded in cache.", console)
+        
+        # Different prompts based on build_index mode
+        if build_index:
+            Message.info("Note: Verbose mode enabled - will build DataFrame index for detailed statistics.", console)
+            should_load = typer.confirm("Would you like to load it with DataFrame indexing now?", default=True)
+        else:
+            should_load = typer.confirm("Would you like to load it now?", default=True)
+        
+        if not should_load:
+            Message.warning(f"Operation cancelled. Please load the bag first using: rose load {bag_path}", console)
+            return False
+    
+    # Load the bag
+    from ..ui.common_ui import CommonUI
+    ui = CommonUI()
+    ui.console = console
+    Message.info("Loading bag file into cache...", console)
+    
+    try:
+        # Use async loading
+        async def load_bag():
+            parser = BagParser()
+            
+            # Create progress callback for loading
+            def progress_callback(current, total, description="Loading"):
+                # Simple progress indication
+                if isinstance(current, (int, float)) and isinstance(total, (int, float)) and total > 0:
+                    percentage = (current / total) * 100
+                    console.print(f"Loading... {percentage:.1f}%", end="\r")
+                else:
+                    # Handle string descriptions
+                    console.print(f"{current}", end="\r")
+            
+            bag_info, elapsed_time = await parser.load_bag_async(
+                str(bag_path), 
+                build_index=build_index,  # Use passed build_index parameter
+                progress_callback=progress_callback if verbose else None
+            )
+            
+            return bag_info, elapsed_time
+        
+        # Run the async loading
+        try:
+            loop = asyncio.get_event_loop()
+        except RuntimeError:
+            loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(loop)
+        
+        bag_info, elapsed_time = loop.run_until_complete(load_bag())
+        
+        if bag_info:
+            Message.success(f"✓ Successfully loaded bag into cache in {elapsed_time:.2f}s", console)
+            if verbose:
+                console.print(f"  Topics: {len(bag_info.topics) if bag_info.topics else 0}")
+                console.print(f"  Duration: {bag_info.duration_seconds:.2f}s" if bag_info.duration_seconds else "  Duration: Unknown")
+            return True
+        else:
+            Message.error("✗ Failed to load bag into cache", console)
+            return False
+            
+    except Exception as e:
+        console.print(f"[red]✗[/red] Error loading bag: {e}")
+        if verbose:
+            import traceback
+            console.print(f"[dim]{traceback.format_exc()}[/dim]")
+        return False
+
