@@ -12,12 +12,14 @@ import logging
 # Import necessary functions from utility modules
 from roseApp.core.util import get_logger, TimeUtil, set_app_mode, AppMode, log_cli_error
 from roseApp.cli.extract import extract as extract_main
+from roseApp.cli.compress import compress as compress_main
 from roseApp.cli.inspect import app as inspect_app
-from roseApp.cli.plot import app as plot_app
-from roseApp.cli.tools import app as tools_app
+# from roseApp.cli.plot import app as plot_app
+from roseApp.cli.cache import app as cache_app
 from roseApp.cli.cli_tool import app as cli_tool_app
+from roseApp.cli.load import load as load_main
 # from roseApp.cli.profile import app as profile_app  # Temporarily disabled due to API migration
-from roseApp.tui.tui import app as tui_app
+# from roseApp.tui.tui import app as tui_app
 
 # Initialize logger
 logger = get_logger("RoseCLI")
@@ -68,19 +70,19 @@ def callback(
         # cache.enable_profiling()  # Enable if profiling method exists
         logger.info("Performance profiling enabled")
     
-    if ctx.invoked_subcommand is None:
-        from .cli.error_handling import show_available_commands
-        show_available_commands()
+
 
 
 # Add subcommands
+app.command(name="load")(load_main)
 app.command(name="extract")(extract_main)
+app.command(name="compress")(compress_main)
 app.add_typer(inspect_app)
-app.add_typer(plot_app)
-app.add_typer(tools_app)
+# app.add_typer(plot_app)
+app.add_typer(cache_app)
 app.add_typer(cli_tool_app)
 # app.add_typer(profile_app)  # Temporarily disabled due to API migration
-app.add_typer(tui_app)
+# app.add_typer(tui_app)
 
 if __name__ == '__main__':
     try:
